/*
 * main.c
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <syslog.h>
#include <libgen.h>
#include <signal.h>

#define DEBUG		0
#include "debug.h"

#define LED_FILE	"/proc/acpi/ibm/led"
#define LED_ID		7

#define BAT_FILE	"/proc/acpi/battery/BAT%d/state"
#define BAT_MAX_LEN	40		// max length of BAT_FILE

#define MAX_LINE_LEN	80

typedef enum {
	OFF, ON, BLINK, HIBERNATE, AC_ONLINE, LED_CMD_MAX
} LED_CMD;

static char *led_cmd[] = { "off", "on", "blink", NULL, "off", NULL };

int set_led (LED_CMD cmd) {
	FILE *f = NULL;
	int ret = 1;
	if (cmd < LED_CMD_MAX && led_cmd[cmd]) {
		if (!(f = fopen(LED_FILE, "w"))) {
			perror(LED_FILE);
			return 0;
		}
		if (fprintf(f, "%d %s\n", LED_ID, led_cmd[cmd]) < 0) {
			perror(LED_FILE);
			ret = 0;
		}
		if (fclose(f) != 0) {
			perror(LED_FILE);
			return 0;
		}
	}
	return ret;
}

/*
 * Returns remaining time in minutes if AC is offline, -1 if AC is online
 * or no battery found.
 */
float read_battery(void) {
	FILE *f = NULL;
	int b, readed = 0;
	char fn[BAT_MAX_LEN];
	int val, present, discharging, capacity, rate;
	int total_capacity = 0, total_rate = 0;
	char line[MAX_LINE_LEN], key[MAX_LINE_LEN], str_val[MAX_LINE_LEN];
	
	for (b = 0;; b++) {
		present = 0;
		discharging = 0;
		snprintf(fn, BAT_MAX_LEN, BAT_FILE, b);
		if ((f = fopen(fn, "r"))) {
			while (fgets(line, MAX_LINE_LEN, f)) {
				sscanf(line, "%[^:]: %s", key, str_val);
				if (isdigit(str_val[0])) {
					val = atoi(str_val);
				} else {
					val = 0;
				}
				DBG_MSG(2, "%s: \"%s\" = \"%s\" (= %d)", fn, key, str_val, val);

				if (strcmp(key, "present") == 0) {
					present = (strcmp(str_val, "yes") == 0);
				} else if (strcmp(key, "charging state") == 0) {
					discharging = (strcmp(str_val, "discharging") == 0);
				} else if (strcmp(key, "remaining capacity") == 0) {
					capacity = val;
					DBG_MSG(2, "capacity += %d", val);
				} else if (strcmp(key, "present rate") == 0) {
					rate = val;
					DBG_MSG(2, "rate += %d", val);
				}
			}
			fclose(f);
			if (discharging) {
				total_capacity += capacity;
				total_rate += rate;
			}
			readed++;
		} else {
			break;
		}
	}
	return readed > 0 && total_rate > 0 ? ((float)total_capacity / (float)total_rate) * 60.0 : -1.0;
}

void on_exit_callback(void) {
	syslog(LOG_INFO, "Exiting.");
	set_led(OFF);
	closelog();
}

void on_signal_callback(int sig) {
	exit(0);
}

int main (int argc, char *argv[]) {
	float rem_time, on_time, blink_time, hibernate_time;
	char *hibernate_cmd;
	LED_CMD old_cmd = LED_CMD_MAX;
	
	if (argc != 5 || !isdigit(argv[1][0]) || !isdigit(argv[2][0]) || !isdigit(argv[3][0])) {
		fprintf(stderr, "Usage: %s on_time blink_time hibernate_time hibernate_command\n"
				"\tEvery time is in minutes.\n", argv[0]);
		return EXIT_FAILURE;
	}

	on_time = atof(argv[1]);
	blink_time = atof(argv[2]);
	hibernate_time = atof(argv[3]);
	hibernate_cmd = argv[4];

	openlog(basename(argv[0]), LOG_PID | (DEBUG > 0 ? LOG_PERROR : 0), LOG_DAEMON);
	syslog(LOG_INFO, "Starting. Times: %0.1f, %0.1f, %0.1f minutes, Command: \"%s\"",
			on_time, blink_time, hibernate_time, hibernate_cmd);
	signal(SIGINT, on_signal_callback);
	signal(SIGTERM, on_signal_callback);
	signal(SIGHUP, SIG_IGN);
	atexit(on_exit_callback);

	for (;;) {
		rem_time = read_battery();
		DBG_MSG(1, "rem_time = %0.3f", rem_time);
		if (rem_time > 0) {
			if (rem_time < hibernate_time) {
				if (old_cmd != HIBERNATE) {
					syslog(LOG_NOTICE, "Time remaining: %0.1f minutes => hibernate !\n", rem_time);
					set_led(HIBERNATE);
					syslog(LOG_NOTICE, "Executing \"%s\"", hibernate_cmd);
					system(hibernate_cmd);
					old_cmd = HIBERNATE;
				}
			} else if (rem_time < blink_time) {
				if (old_cmd != BLINK) {
					syslog(LOG_NOTICE, "Time remaining: %0.1f minutes => blink !\n", rem_time);
					set_led(BLINK);
					old_cmd = BLINK;
				}
			} else if (rem_time < on_time) {
				if (old_cmd != ON) {
					syslog(LOG_NOTICE, "Time remaining: %0.1f minutes => turn LED on !\n", rem_time);
					set_led(ON);
					old_cmd = ON;
				}
			} else {
				if (old_cmd != OFF) {
					syslog(LOG_NOTICE, "Time remaining: %0.1f minutes => turn LED off.\n", rem_time);
					set_led(OFF);
					old_cmd = OFF;
				}
			}
		} else {
			if (old_cmd != AC_ONLINE) {
				syslog(LOG_NOTICE, "AC online => turn LED off.\n");
				set_led(AC_ONLINE);
				old_cmd = AC_ONLINE;
			}
		}
		sleep(10);
	}

	return EXIT_SUCCESS;
}


